@@ -1,5 +1,5 @@
 /*
- * MUSCLE SmartCard Development ( http://pcsclite.alioth.debian.org/pcsclite.html )
+ * MUSCLE SmartCard Development ( https://pcsclite.apdu.fr/ )
  *
  * Copyright (C) 1999-2004
  *  David Corcoran <corcoran@musclecard.com>
@@ -348,8 +348,8 @@ LONG SCardConnect(/*@unused@*/ SCARDCONTEXT hContext, LPCSTR szReader,
 		}
 
 		/* the card is now in use */
-		rContext->powerState = POWER_STATE_INUSE;
-		Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_INUSE");
+		rContext->powerState = POWER_STATE_IN_USE;
+		Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_IN_USE");
 		(void)pthread_mutex_unlock(&rContext->powerState_lock);
 	}
 
@@ -716,6 +716,12 @@ LONG SCardReconnect(SCARDHANDLE hCard, DWORD dwShareMode,
 					goto exit;
 				}
 			}
+
+			/* the card is now in use */
+			(void)pthread_mutex_lock(&rContext->powerState_lock);
+			rContext->powerState = POWER_STATE_IN_USE;
+			Log1(PCSC_LOG_DEBUG, "powerState: POWER_STATE_IN_USE");
+			(void)pthread_mutex_unlock(&rContext->powerState_lock);
 		}
 	}
 
